using Microsoft.AspNetCore.Builder;
using Microsoft.AspNetCore.Hosting;
using Microsoft.Extensions.Configuration;
using Microsoft.Extensions.Hosting;
using NetworkResurrector.Server.Extensions;
using Serilog;
using System;

namespace NetworkResurrector.Server
{
    public class Program
    {
        public static void Main(string[] args)
        {
            var builder = WebApplication.CreateBuilder(args);

            builder.Host.UseSerilog((_, lc) =>
            {
                lc
                .ReadFrom.Configuration(builder.Configuration)
                .Enrich.FromLogContext()
                .WriteTo.Console()
                .WriteTo.ConfiguredDestinations(builder.Configuration);
            });

            builder.Services.ConfigureServices(builder.Configuration);

            var useWindowsService = builder.Configuration.GetValue<bool>("Host:UseWindowsService");
            if (useWindowsService)
                builder.Host.UseWindowsService();

            var app = builder.Build();
            app.Configure();

            var exitCode = 0;

            try
            {
                var urls = builder.Configuration.GetValue<string>("Urls");
                Log.Information("Starting network resurrector server...");
                Log.Information($"Server listening on {urls}");
                Console.WriteLine("Application started. Press Ctrl+C to shut down.");
                app.Run();
            }
            catch (Exception ex)
            {
                Log.Fatal(ex, "Network resurrector server host terminated unexpectedly");
            }
            finally
            {
                Log.CloseAndFlush();
            }

            Environment.Exit(exitCode);
        }
    }
}
